/* ☰☰☰☰☰☰☰☰☰☰☰☰☰☰☰☰☰☰☰☰☰☰☰☰☰☰☰☰☰☰☰☰☰☰☰☰☰☰☰☰☰☰☰☰☰☰☰☰☰☰☰☰☰☰☰☰☰☰☰☰☰☰☰☰☰☰☰☰☰☰☰☰☰☰☰☰☰☰☰☰☰☰☰☰☰☰
 *  
 * Levitate: Contact-sensitive Prosthetic Device for Individuals with Impaired Tactile Sensation.
 * Designed & made open-source by Akshay Dinakar.
 * 
 * Holistic: This prosthetic device uses a velostat sensor (attached to any relevant part of the body) 
 * to read in pressure values via Analog Pins on an appropriate microcontroller (recommended: any Arduino
 * with a boost converter, or Adafruit Flora / Feather [which come with built-in li-po battery charging via
 * USB cable]). Once the pressure value reaches a certain lower limit (the "CUTOFF" - specified by the user, 
 * based on the sensitivity of the velostat), a specified haptic signal is activated by the attached motor
 * driver (recommended: DRV3605L in conjuction with an LRA vibration motor [2-3V range]).
 * 
 * For hardware files / assembly instructions, please see the full tutorial on Instructables.
 * 
 * ☰☰☰☰☰☰☰☰☰☰☰☰☰☰☰☰☰☰☰☰☰☰☰☰☰☰☰☰☰☰☰☰☰☰☰☰☰☰☰☰☰☰☰☰☰☰☰☰☰☰☰☰☰☰☰☰☰☰☰☰☰☰☰☰☰☰☰☰☰☰☰☰☰☰☰☰☰☰☰☰☰☰☰☰☰☰☰
 */

#include <Wire.h>
#include "Adafruit_DRV2605.h"

// Initializing the haptic motor driver.
Adafruit_DRV2605 drv;

// Edit these values based on your velostat sensitivity.
int CUTOFF = 20; // CUTOFF is the limit for haptic feedback activation.
int CLIFF = 40; // CLIFF is a slightly higher value to allow pressure to "lift off" of the velostat layer (serves as buffer)

void setup() {
  Serial.begin(9600);

  // Sets up the analog pin to read in pressure values from the velostat sensor.
  pinMode(A1, INPUT_PULLUP); // Be sure to connect your velostat input to the appropriate Analog pin (A1) on your microcontroller.
  digitalWrite(A1, HIGH);
  
  // Sets up the haptic motor driver.
  drv.begin();
  drv.selectLibrary(1); // Calls access to the library of pre-built vibration patterns.
  drv.setMode(DRV2605_MODE_INTTRIG); // I2C Trigger
  drv.useLRA(); // Instructs the haptic motor driver that an LRA vibration motor is being used (toggles AC vs DC signal).
}

void loop() {
  int sensorValue = analogRead(A1); // Reads in the current pressure sensor value.
  Serial.println(sensorValue);
  
  if (sensorValue < CUTOFF) {
    drv.setWaveform(0, 47); // Standard buzz waveform.
    drv.setWaveform(1, 0); // Ends the vibration waveform.

    drv.go(); // Plays the waveform we have created.
    delay(500);

    while (analogRead(A0) < CLIFF) { // Sets buffer to ensure motor does not continuously buzz while pressure sensor is activated.
      Serial.println((analogRead(A0)));
      delay(50);
    }
  }
}
